% Fitting initial LIBOR rates and
% closed form pricing of caplets in the CIR model.

function main=affine_libor_CIR_closed_caplets_20090313()

global lambda theta eta Xzero TN u

tic;

% CIR parameters ::
 lambda = 0.001;
 theta = 0.50;
 eta = 0.350;
 Xzero = 1.2500;

% market parameters ::
 T = [0.5,1.0,1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0];
 Tstar = 0.5:0.5:4.5;
 N = length(T);
 TN = T(N);
 delta = 0.50;
 Kseries=0.010:0.005:0.065;
 K = 1+delta*Kseries;
 Bonds = [0.9833630,0.9647388,0.9435826,0.9228903,0.9006922,0.8790279,0.8568412,0.8352144,0.8133497,0.7920573];
 BQuot = Bonds./Bonds(N)

% tests
 test1 = zeros(1,N-1);
 test2 = zeros(1,N-1);
 test3 = zeros(1,N-1);
 test4 = zeros(1,N-1);
 test5 = zeros(1,N-1);
 C = zeros(N-1,N);
 IV = zeros(N-1,N);

% Functions for the CIR model ::
function tmp = ai(t)
  tmp = exp(-lambda*t);
end

function tmp = bi(t)
  tmp = (1-exp(-lambda*t))/lambda;
end

function tmp = phi(t,v)
  tmp = -lambda*theta*log(1-2*eta*v*bi(t))/(2*eta);
end

function tmp = psi(t,v)
  tmp = ai(t)*v/(1-2*eta*v*bi(t));
end

function tmp = A(w,y)
  tmp = w-y;
end

function tmp = B(w,y)
  tmp = w-y;
end

% Functions for the fitting of the initial tenor structure of LIBOR rates ::
function tmp = Mini(w)
  tmp = exp(phi(TN,w)+psi(TN,w)*Xzero);
end

function tmp = Diff(v,k)
  tmp = Mini(v)-BQuot(k);
end

% Function for testing ::
function tmp = BondQuot(w,y)
  tmp = exp(phi(TN,w)-phi(TN,y)+(psi(TN,w)-psi(TN,y))*Xzero);
end

% Functions for closed form pricing

B0 = Bonds;

for(k=1:(N))
  u(k) = fzero(@(v)Diff(v,k),0.05);
end

for(k=1:(N-1))
 dim = lambda*theta/eta;
 zeta1 = 1-2*eta*bi(T(k))*psi(TN-T(k),u(k));
 zeta2 = 1-2*eta*bi(T(k))*psi(TN-T(k),u(k+1));
 sigma1 = B(psi(TN-T(k),u(k)),psi(TN-T(k),u(k+1)))*eta*bi(T(k))/zeta1;
 sigma2 = B(psi(TN-T(k),u(k)),psi(TN-T(k),u(k+1)))*eta*bi(T(k))/zeta2;
 alpha1 = Xzero*ai(T(k))/(eta*bi(T(k))*zeta1);
 alpha2 = Xzero*ai(T(k))/(eta*bi(T(k))*zeta2);

  test4(k) = Mini(u(k))-BQuot(k);

  L0(k) = 1/delta*(BondQuot(u(k),u(k+1))-1);
  L1(k) = 1/delta*(Bonds(k)/Bonds(k+1)-1);
  L2(k) = 1/delta*(Mini(u(k))/Mini(u(k+1))-1);

  for(l=1:length(K))

   arg1 = (log(K(l))-A(phi(TN-T(k),u(k)),phi(TN-T(k),u(k+1))))/sigma1;
   arg2 = (log(K(l))-A(phi(TN-T(k),u(k)),phi(TN-T(k),u(k+1))))/sigma2;

   C(k,l) = B0(k)*(1-ncx2cdf(arg1,dim,alpha1))-B0(k+1)*K(l)*(1-ncx2cdf(arg2,dim,alpha2));
   IV(k,l) = ivol(L0(k),Kseries(l),T(k+1),0.0,real(C(k,l))/(delta*B0(k+1)))*sqrt(T(k+1)/T(k));
  end
end

test4
L0
L1
L2
u
10000*real(C)
IV
surf(Kseries,Tstar,IV)

toc;

end