% Fitting initial LIBOR rates and
% Fourier-pricing of caplets in the Gamma-OU model.

function main=affine_libor_GammaOU_fourier_caplets_20090320()

global lambda alpha beta Xzero TN R u

tic;

 R = 3.25;

% CIR parameters ::
 lambda = 0.0100;
 alpha = 2.00;
 beta = 5.00;
 Xzero = 1.0;

% market parameters ::
 T = [0.5,1.0,1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0];
 Tstar = 0.5:0.5:4.5;
 N = length(T);
 TN = T(N);
 delta = 0.50;
 Kseries=0.025:0.005:0.070;
 K = 1+delta*Kseries;
 Bonds = [0.9833630,0.9647388,0.9435826,0.9228903,0.9006922,0.8790279,0.8568412,0.8352144,0.8133497,0.7920573];
 BQuot = Bonds./Bonds(N)

% tests
 test1 = zeros(1,N-1);
 test2 = zeros(1,N-1);
 test3 = zeros(1,N-1);
 test4 = zeros(1,N-1);
 test5 = zeros(1,N-1);
 C = zeros(N-1,N);
 IV = zeros(N-1,N);

% Functions for the CIR model :: ("mgf" is called \Lambda in the paper, "aux" does NOT exist in the paper)
function tmp = ai(t)
  tmp = exp(-lambda*t);
end

function tmp = phi(t,v)
  tmp = beta*log((alpha-ai(t)*v)/(alpha-v));
end

function tmp = psi(t,v)
  tmp = ai(t)*v;
end

function tmp = aux(t,v,w,y)
  tmp = v*psi(TN-t,w) + (1-v)*psi(TN-t,y);
end

function tmp = mgf(v,t,w,y)
  tmp = exp(v*phi(TN-t,w)+(1-v)*phi(TN-t,y)+phi(t,aux(t,v,w,y))+psi(t,aux(t,v,w,y))*Xzero)*exp(-phi(TN,y)-psi(TN,y)*Xzero);
end


% Function for the integrand of the option price ::
function tmp = fun(v,t,w,y,strike)
  tmp = strike.^complex(-R,v)./(complex(R,-v).*complex(R-1,-v)).*mgf(complex(R,-v),t,w,y);
end

% Functions for the fitting of the initial tenor structure of LIBOR rates ::
function tmp = Mini(w)
  tmp = exp(phi(TN,w)+psi(TN,w)*Xzero);
end

function tmp = Diff(v,k)
  tmp = Mini(v)-BQuot(k);
end

% Function for testing ::
function tmp = BondQuot(w,y)
  tmp = exp(phi(TN,w)-phi(TN,y)+(psi(TN,w)-psi(TN,y))*Xzero);
end

B0 = Bonds;

for(k=1:N)
  u(k) = fzero(@(v)Diff(v,k),0.05);
end

for(k=1:(N-1))
  test1(k) = (alpha-ai(TN-T(k))*u(k))/(alpha-u(k));
  test2(k) = (alpha-ai(TN-T(k))*u(k+1))/(alpha-u(k+1));
  test3(k) = (alpha-ai(T(k))*aux(T(k),R,u(k),u(k+1)))/(alpha-aux(T(k),R,u(k),u(k+1)));
  test4(k) = Mini(u(k))-BQuot(k);
  test5(k) = mgf(0,T(k),u(k),u(k+1));

  L0(k) = 1/delta*(BondQuot(u(k),u(k+1))-1);
  L1(k) = 1/delta*(Bonds(k)/Bonds(k+1)-1);
  L2(k) = 1/delta*(Mini(u(k))/Mini(u(k+1))-1);

  for(l=1:length(K))
   C(k,l) = B0(k+1)*K(l)/(2*pi)*quad(@(v)fun(v,T(k),u(k),u(k+1),K(l)),-1000000,1000000); %,0.00000001);
   IV(k,l) = ivol(L0(k),Kseries(l),T(k+1),0.0,real(C(k,l))/(delta*B0(k+1)))*sqrt(T(k+1)/T(k));
  end
end

test1
test2
test3
test4
test5
L0
L1
L2
u
10000*real(C)
IV
surf(Kseries,Tstar,IV)

toc;

end