% Fitting initial LIBOR rates and
% Fourier-pricing of caplets in the Gamma-OU model.

function main=libor_black_20101124()

global Xzero TN R

tic;

 R = 3.25;

% Black parameters ::
 si=[0.1,0.12,0.12,0.12,0.12,0.12,0.12,0.12,0.12];

% market parameters ::
 T = [0.5,1.0,1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0];
 Tstar = 0.5:0.5:4.5;
 N = length(T)
 TN = T(N);
 delta = 0.50;
 Kseries=0.025:0.005:0.070;
 K = 1+delta*Kseries;
 M=length(K)
 Bonds = [0.9833630,0.9647388,0.9435826,0.9228903,0.9006922,0.8790279,0.8568412,0.8352144,0.8133497,0.7920573];
 BQuot = Bonds./Bonds(N)

% tests
 C = zeros(N-1,M);
 IV = zeros(N-1,M);

% Functions for the Black model :: 
function tmp=bsc(S,K,T,r,si)
  d1=(log(S/K)+(r+0.5*si^2)*T)/(si*sqrt(T));
  d2=(log(S/K)+(r-0.5*si^2)*T)/(si*sqrt(T));
  N1=normcdf(d1);
  N2=normcdf(d2);
  tmp=S*N1-K*exp(-r*T)*N2;
end


% Functions for closed form pricing

B0 = Bonds;

for(k=1:(N-1))
   L1(k) = 1/delta*(Bonds(k)/Bonds(k+1)-1);
  for(l=1:M)
   C(k,l) = delta*B0(k+1)*bsc(L1(k),Kseries(l),T(k+1),0,si(k));
   IV(k,l) = ivol(L1(k),Kseries(l),T(k+1),0.0,real(C(k,l))/(delta*B0(k+1)))*sqrt(T(k+1)/T(k));
  end
end

10000*real(C)
IV
surf(Kseries,Tstar,IV)

toc;

end